import numpy as np
import face_recognition as fr
from cv2 import cv2
import pickle
from datetime import datetime
import time
import dlib
from pygame import mixer
from parameters import *
from imutils import face_utils as face
from scipy.spatial import distance
from flask import Flask
from flask import request
from flask_cors import CORS
import ffmpeg    
import db
import json, os, signal

app  = Flask("__main__")
CORS(app)

encodingsFile: str = r"C:\\Users\Asus\Desktop\\Research\\New\\Component1\\feceRecognition\\encoding.pickle"

current_path = os.path.abspath(os.path.join(os.path.dirname(__file__)))
video_path = current_path+"\\resources\\testt.mp4"

def check_rotation():
     # this returns meta-data of the video file in form of a dictionary
     meta_dict = ffmpeg.probe(video_path)

     # from the dictionary, meta_dict['streams'][0]['tags']['rotate'] is the key
     # we are looking for
     rotateCode = None
     if int(meta_dict['streams'][0]['tags']['rotate']) == 90:
         rotateCode = cv2.ROTATE_90_CLOCKWISE
     elif int(meta_dict['streams'][0]['tags']['rotate']) == 180:
         rotateCode = cv2.ROTATE_180
     elif int(meta_dict['streams'][0]['tags']['rotate']) == 270:
         rotateCode = cv2.ROTATE_90_COUNTERCLOCKWISE

     return rotateCode

def correct_rotation(frame, rotateCode):  
     return cv2.rotate(frame, rotateCode) 

# attendance details send to database
def MarkAttendence(name,id):
    now = datetime.now()
    dt = now.strftime('%H:%M:%S')
    con = db.connection()
    cursor = con.cursor()
    select_data = "select count(*) from attendence where name='%s'"%name
    cursor.execute(select_data)
    result = cursor.fetchall()

    if(result[0][0] == 1):
        sql = "UPDATE attendence SET time=%s, sub=%s WHERE name =%s"
        val = [str(dt),id,name]
        cursor.execute(sql,val)
        con.commit()
    else:
        sql = "INSERT INTO attendence (name, time, sub) VALUES (%s, %s, %s)" 
        val = [name,str(dt),id]
        cursor.execute(sql,val)
        con.commit()

def get_max_area_rect(rects):
    if len(rects)==0: return
    areas=[]
    for rect in rects:
        areas.append(rect.area())
    return rects[areas.index(max(areas))]

def get_eye_aspect_ratio(eye):
    vertical_1 = distance.euclidean(eye[1], eye[5])
    vertical_2 = distance.euclidean(eye[2], eye[4])
    horizontal = distance.euclidean(eye[0], eye[3])
    return (vertical_1+vertical_2)/(horizontal*2) #aspect ratio of eye

def get_mouth_aspect_ratio(mouth):
    horizontal=distance.euclidean(mouth[0],mouth[4])
    vertical=0
    for coord in range(1,4):
        vertical+=distance.euclidean(mouth[coord],mouth[8-coord])
    return vertical/(horizontal*3) #mouth aspect ratio


@app.route('/detectFaces', methods=['GET']) 
def main():
    id = request.args.get('id')
    time.sleep(10)
    video_capture = cv2.VideoCapture(1)
    # check if video requires rotation
    #rotateCode = cv2.ROTATE_90_CLOCKWISE
    # load the known faces and embeddings
    print("[INFO] loading encodings...")
    data = pickle.loads(open(encodingsFile, "rb").read())

    # model read
    known_face_encoding = data["encodings"]
    known_face_names = data["names"]

    mixer.init()
    distracton_initlized = False
    eye_initialized      = False
    mouth_initialized    = False

    detector    = dlib.get_frontal_face_detector()
    predictor   = dlib.shape_predictor(shape_predictor_path)

    ls,le = face.FACIAL_LANDMARKS_IDXS["left_eye"]
    rs,re = face.FACIAL_LANDMARKS_IDXS["right_eye"]

    fps_couter = 0
    fps_to_display = 'initializing..'
    fps_timer = time.time()

    while True:

        if id == "stop":
            break

        ret, frame = video_capture.read()

        fps_couter+=1
        frame_new = cv2.flip(frame,1)
        if time.time()-fps_timer>=1.0:
            fps_to_display=fps_couter
            fps_timer=time.time()
            fps_couter=0

        # change the colors of frame to RGB
        rgb_frame = frame [:, :, ::-1] 

        face_location = fr.face_locations(rgb_frame)
        face_encodings = fr.face_encodings(rgb_frame)

        for (top, right, bottom, left), face_encoding in zip(face_location, face_encodings):
            matches = fr.compare_faces(known_face_encoding, face_encoding)

            name = "Unknown"
            face_distances = fr.face_distance(known_face_encoding, face_encoding)
            best_match_index = np.argmin(face_distances)
        
            #print(known_face_encoding[best_match_index])
            if matches[best_match_index]:
                name = known_face_names[best_match_index]
                #attendance saved
                MarkAttendence(name,id)
            print(name)
            # frame over face. rectangle vertices, frame color, frame thickness
            cv2.rectangle(frame, (left, top), (right, bottom), (0,0,255), 2)
            # frame to display name
            cv2.rectangle(frame, (left, bottom -35), (right, bottom), (0, 0, 255), cv2.FILLED)
            font = cv2.FONT_HERSHEY_SIMPLEX

        gray = cv2.cvtColor(frame_new, cv2.COLOR_BGR2GRAY)

        rects = detector(gray, 0)
        rect=get_max_area_rect(rects)

        if rect!=None:

            distracton_initlized=False

            shape = predictor(gray, rect)
            shape = face.shape_to_np(shape)

            leftEye = shape[ls:le]
            rightEye = shape[rs:re]
            leftEye = get_eye_aspect_ratio(leftEye)
            rightEye = get_eye_aspect_ratio(rightEye)

            inner_lips=shape[60:68]
            mar=get_mouth_aspect_ratio(inner_lips)

            eye_aspect_ratio = (leftEye + rightEye) / 2.0

            if eye_aspect_ratio < EYE_DROWSINESS_THRESHOLD:

                if not eye_initialized:
                    eye_start_time= time.time()
                    eye_initialized=True

                if time.time()-eye_start_time >= EYE_DROWSINESS_INTERVAL:
                    alarm_type=0

                    if  not distracton_initlized and not mouth_initialized and not mixer.music.get_busy():
                        mixer.music.load(alarm_paths[alarm_type])
                        mixer.music.play()
            else:
                eye_initialized=False
                if not distracton_initlized and not mouth_initialized and mixer.music.get_busy():
                    mixer.music.stop()


            if mar > MOUTH_DROWSINESS_THRESHOLD:

                if not mouth_initialized:
                    mouth_start_time= time.time()
                    mouth_initialized=True

                if time.time()-mouth_start_time >= MOUTH_DROWSINESS_INTERVAL:
                    alarm_type=2

                    if not mixer.music.get_busy():
                        mixer.music.load(alarm_paths[alarm_type])
                        mixer.music.play()
            else:
                mouth_initialized=False
                if not distracton_initlized and not eye_initialized and mixer.music.get_busy():
                    mixer.music.stop()
        else:
            alarm_type=1
            if not distracton_initlized:
                distracton_start_time=time.time()
                distracton_initlized=True

            if time.time()- distracton_start_time> DISTRACTION_INTERVAL:

                if not eye_initialized and not mouth_initialized and not  mixer.music.get_busy():
                    mixer.music.load(alarm_paths[alarm_type])
                    mixer.music.play()


        #cv2.imshow('Webcam_facerecognition', frame)

        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    video_capture.release()
    cv2.destroyAllWindows()

@app.route('/login', methods=['GET']) 
def login():
    uname = request.args.get('uname')
    pwd = request.args.get('pwd')
        
    con = db.connection()
    cursor = con.cursor()
    selectData = "select * from users where uname='%s'"%uname #retrieve data from mysql db
    cursor.execute(selectData)   
    result = cursor.fetchall()
    print(result[0])
    user = False

    if(result[0][1] == pwd):
        user = True

    values = {
        "user" : user,
        "type" : result[0][2]
    }

    return values

@app.route('/attendence', methods=['GET']) 
def attendence():
    sub = request.args.get('id')

    con = db.connection()
    cursor = con.cursor()
    selectData = "select name from attendence where sub='%s'"%sub #retrieve data from mysql db
    cursor.execute(selectData)   
    result = cursor.fetchall()
    print(result)
  
    values = {
        "user" : result,
        "size" : len(result)
    }

    return values

@app.route('/stopServer')
def stopServer():
    os.kill(os.getpid(), signal.SIGINT)
    return "Server is shutting down..."

#if __name__ == '__main__':
   # MarkAttendence("user","1")